@@ -1,3 +1,4 @@
+import base64
 import json
 import re
 import socket
@@ -6,11 +7,10 @@ from http import HTTPStatus
 from typing import Union
 from urllib.parse import urljoin, urlencode
 
+import plogger
 import requests
 from urllib3 import disable_warnings, exceptions
 
-from .logger import logger
-
 disable_warnings(exceptions.InsecureRequestWarning)
 
 
@@ -27,19 +27,6 @@ def get_header_default(user_agent='ide', content_type='application/json-patch+js
     }
 
 
-def log_params(func):
-    def inner(*args, **kwargs):
-        # log_info = f'{locals()["kwargs"]}.'
-
-        log_info = ' | '.join([f'{key}={value}' for key, value in locals()["kwargs"].items()])
-        if len(args) > 1:
-            log_info += f'UNKNOWN params: {args[1:]}'
-        logger.info(log_info)
-        return func(*args, **kwargs)
-
-    return inner
-
-
 class RESTAssertion:
     """Parse requests response to get status code."""
 
@@ -68,8 +55,8 @@ class RESTClient(RESTAssertion):
     """ Main class for RestAPI """
 
     def __init__(self,
-                 protocol: str = '',
-                 host: str = '',
+                 protocol: str = 'https',
+                 host: str = None,
                  port: int = 0,
                  username: str = None,
                  password: str = None,
@@ -77,21 +64,22 @@ class RESTClient(RESTAssertion):
                  auth_basic: bool = False,
                  auth_uri: str = '',
                  auth_payload: dict = None,
+                 auth_timeout: int = 7,
                  logger_enabled: bool = True):
 
         # Enable/disable logger
-        self.logger = logger
-        logger.disabled = not logger_enabled
+        self.logger = plogger.logger(self.__class__.__name__, enabled=logger_enabled)
 
         # Credentials and settings for api url
         self.username = username
         self.password = password
         self.protocol = protocol  # HTTP or HTTPS
-        self.host = host
+        self.host = host if host is not None else '127.0.0.1'
         self.port = port
         self.auth_uri = auth_uri
         self.auth_payload = auth_payload
         self.auth_basic = auth_basic
+        self.auth_timeout = auth_timeout
         self.header = header or get_header_default()  # Default header
 
     def __str__(self):
@@ -119,6 +107,21 @@ class RESTClient(RESTAssertion):
     def _validate_url(url):
         return re.match(r'http[s]?://[\w/.]+', url)
 
+    def encode_to_base64(self, string: str):
+        """Encode string to base64
+
+        :param string:
+        :return:
+        """
+
+        try:
+            str_text = string.encode()
+            base64_encoded = base64.b64encode(str_text)
+            return base64_encoded.decode()
+        except AttributeError as err:
+            self.logger.exception(f'Cannot encode specific data ({string}, {type(string)})')
+            raise err
+
     def get_token(self) -> str:
         """Get auth (Bearer) token"""
 
@@ -141,7 +144,7 @@ class RESTClient(RESTAssertion):
             self.header['Authorization'] = self.token
             return self.header
         except KeyError as err:
-            logger.error(f'Cannot get token. {err}')
+            self.logger.error(f'Cannot get token. {err}')
 
     def _auth_basic(self):
         """Enable basic auth if provided"""
@@ -202,7 +205,7 @@ class RESTClient(RESTAssertion):
             'Specified methods is not compatible.'
 
         if data and files:
-            logger.error('Data parameter cannot be used with files parameter simultaneously.')
+            self.logger.error('Data parameter cannot be used with files parameter simultaneously.')
             raise ValueError('Data parameter cannot be used with files parameter simultaneously.')
 
         # Extend header
@@ -211,17 +214,15 @@ class RESTClient(RESTAssertion):
 
         response = 'Invalid requests. Check parameters'
 
-        logger.info(f'[{method}] {link}')
+        self.logger.info(f'[{method}] {link}')
 
         try:
             # For 'POST', 'PUT', 'PATCH', 'OPTIONS', 'DELETE'
             if method in ['POST', 'PUT', 'PATCH', 'OPTIONS', 'DELETE']:
                 if data and not files:
 
-                    logger.debug(f'Headers={self.header}, '
-                                 f'Cookies={cookies}, '
-                                 f'Timeout={request_timeout}')
-                    logger.info('PAYLOAD:\n' + json.dumps(data, indent=4))
+                    self.logger.debug(f'{self.header = }, {cookies = }, {request_timeout = }')
+                    self.logger.info('PAYLOAD:\n' + json.dumps(data, indent=4))
 
                     response = requests.request(method=method,
                                                 url=link,
@@ -236,10 +237,8 @@ class RESTClient(RESTAssertion):
                     files_header = self.header.copy()
                     del files_header['Content-Type']
 
-                    logger.debug(f'Headers={files_header}, '
-                                 f'Cookies={cookies}, '
-                                 f'Timeout={request_timeout}')
-                    logger.info('PAYLOAD:\n' + json.dumps(data, indent=4))
+                    self.logger.debug(f'{files_header = }, {cookies = }, {request_timeout = }')
+                    self.logger.info('PAYLOAD:\n' + json.dumps(data, indent=4))
 
                     response = requests.request(method=method,
                                                 url=link,
@@ -251,9 +250,7 @@ class RESTClient(RESTAssertion):
                                                 timeout=request_timeout)
 
                 elif not files and not data:
-                    logger.debug(f'Header={self.header}, '
-                                 f'Cookies={cookies}, '
-                                 f'Timeout={request_timeout}')
+                    self.logger.debug(f'{self.header = }, {cookies = }, {request_timeout = }')
 
                     response = requests.request(method=method,
                                                 url=link,
@@ -266,9 +263,7 @@ class RESTClient(RESTAssertion):
             # For 'GET', 'HEAD' request
             else:
                 # For usual 'GET' without query params
-                logger.debug(f'Header={self.header}, '
-                             f'Cookies={cookies}, '
-                             f'Timeout={request_timeout}')
+                self.logger.debug(f'{self.header = }, {cookies = }, {request_timeout = }')
 
                 response = requests.request(method=method,
                                             url=link,
@@ -279,7 +274,7 @@ class RESTClient(RESTAssertion):
                                             timeout=request_timeout)
 
         except BaseException as err:
-            logger.exception('Something wend wrong.')
+            self.logger.exception('Something wend wrong.')
             raise err
 
         return response
@@ -306,7 +301,7 @@ class RESTClient(RESTAssertion):
         :return:
         """
 
-        response = self.send_request('GET', url)
+        response = self.GET(url)
         try:
             if response.ok:
                 with open(dst, 'wb') as f:
@@ -315,7 +310,7 @@ class RESTClient(RESTAssertion):
             else:
                 return False
         except (ConnectionError, ConnectionRefusedError) as err:
-            logger.error(f'Download failed. {err}')
+            self.logger.exception('Download failed')
             return err
 
     # noinspection PyPep8Naming
@@ -453,18 +448,21 @@ class AuthToken:
         response_json = None
 
         try:
-            response = requests.post(self.auth_url, json=self.client.auth_payload, verify=False, timeout=7)
+            payload = self.client.auth_payload
+            timeout = self.client.auth_timeout
+            response = requests.post(self.auth_url, json=payload, verify=False, timeout=timeout)
             response_json = response.json()
             return response_json['token']
         except KeyError as err:
-            logger.exception(f'Cannot get key (token) from json data. {response_json}')
+            self.client.logger.exception(f'Cannot get key (token) from json data. {response_json}')
             raise err
         except json.decoder.JSONDecodeError as err:
-            logger.exception(f'Cannot get token property. URL: {self.auth_url} ({self.client.auth_payload})')
+            msg = f'Cannot get token property. URL: {self.auth_url} ({self.client.auth_payload})'
+            self.client.logger.exception(msg)
             raise err
         except TypeError as err:
-            logger.exception('Endpoint is available but "token" key cannot be retrieved.')
+            self.client.logger.exception('Endpoint is available but "token" key cannot be retrieved.')
             raise err
         except requests.exceptions.ConnectTimeout as err:
-            logger.exception(f'Perhaps, URI ({self.auth_url}) or service is unreachable.')
+            self.client.logger.exception(f'Perhaps, URI ({self.auth_url}) or service is unreachable.')
             raise err
